//-------+---------+---------+---------+---------+---------+---------+--------=
//
// File: genpaf.h
//
//----------

#ifndef genpaf_H				// (prevent multiple inclusion)
#define genpaf_H

// other files

#include <stdio.h>				// standard C i/o stuff
#include <stdarg.h>				// standard C variable argument list stuff
#include "utilities.h"			// utility stuff
#include "sequences.h"			// sequence stuff
#include "edit_script.h"		// alignment edit script stuff

// establish ownership of global variables

#ifdef genpaf_owner
#define global
#else
#define global extern
#endif

// type codes for printable fields

enum
	{
	genpafName1            = 'N',
	genpafStrand1          = 'D',
	genpafSize1            = 'S',
	genpafStart1           = 'B',
	genpafStart1Zero       = 'Z',
	genpafStart1DotPlot    = '0',
	genpafStart1Cigar      = '5',
	genpafEnd1             = 'E',
	genpafEnd1DotPlot      = '1',
	genpafEnd1Cigar        = '6',
	genpafLength1          = 'L',
	genpafText1            = 'T',
	genpafName2            = 'n',
	genpafStrand2          = 'd',
	genpafSize2            = 's',
	genpafStart2           = 'b',
	genpafStart2Zero       = 'z',
	genpafStart2OnPlus     = '<',
	genpafStart2ZeroOnPlus = '>',
	genpafStart2DotPlot    = '2',
	genpafStart2Cigar      = '7',
	genpafEnd2             = 'e',
	genpafEnd2OnPlus       = ',',
	genpafEnd2DotPlot      = '3',
	genpafEnd2Cigar        = '8',
	genpafLength2          = 'l',
	genpafText2            = 't',
	genpafTextDiff         = '=',
	genpafCigar            = '&',
	genpafDiagonal         = '/',
	genpafShingle          = '\\',
	genpafScore            = '#',
	genpafIdentity         = 'I',
	genpafCoverage         = 'C',
	genpafGapRate          = 'G',
	genpafNA               = 'X',
	genpafMarker           = '~',
	genpafCR               = '!',
	genpafInfoSeparator    = ';'
	};

#define genpafStandardKeys "#NDSZEndszeIC"
#define genpafRDotPlotKeys "02!13!XX"

#define genpafTDName        "diff"
#define genpafTDInfoDefault ".:x--X"	// (indexed by these next definitons)
#define genpafTDInfoMatch        0
#define genpafTDInfoTransition   1
#define genpafTDInfoTransversion 2
#define genpafTDInfoInsert1      3
#define genpafTDInfoInsert2      4
#define genpafTDInfoOther        5
#define genpafTDInfoSize         6

typedef struct
	{
	char*	name;
	char	key;
	} stringtokey;

#ifdef genpaf_owner
global stringtokey genpafName[] =
	{
	{ "name1",      genpafName1            },
	{ "strand1",    genpafStrand1          },
	{ "size1",      genpafSize1            },
	{ "start1",     genpafStart1           },
	{ "zstart1",    genpafStart1Zero       },
	{ "end1",       genpafEnd1             },
	{ "length1",    genpafLength1          },
	{ "text1",      genpafText1            },
	{ "name2",      genpafName2            },
	{ "strand2",    genpafStrand2          },
	{ "size2",      genpafSize2            },
	{ "start2",     genpafStart2           },
	{ "zstart2",    genpafStart2Zero       },
	{ "start2+",    genpafStart2OnPlus     },
	{ "zstart2+",   genpafStart2ZeroOnPlus },
	{ "end2",       genpafEnd2             },
	{ "end2+",      genpafEnd2OnPlus       },
	{ "length2",    genpafLength2          },
	{ "text2",      genpafText2            },
	{ genpafTDName, genpafTextDiff         },
	{ "cigar",      genpafCigar            },
	{ "diagonal",   genpafDiagonal         },
	{ "shingle",    genpafShingle          },
	{ "score",      genpafScore            },
	{ "identity",   genpafIdentity         },
	{ "coverage",   genpafCoverage         },
	{ "gaprate",    genpafGapRate          },
	{ "NA",         genpafNA               },
	{ "~",          genpafMarker           },
	{ NULL,         0                      }  // (list terminator)
	};
#else
global stringtokey genpafName[];
#endif

//----------
//
// prototypes for routines in genpaf.c
//
//----------

void print_genpaf_job_header (FILE* f, char* keys);
void print_genpaf_job_footer (FILE* f);
void print_genpaf_header     (FILE* f, seq* seq1, seq* seq2);
void print_genpaf_align_list (FILE* f, alignel* alignList, seq* seq1, seq* seq2,
                              char* keys);
void print_genpaf_align_list_segments
                             (FILE* f, alignel* alignList, seq* seq1, seq* seq2,
                              char* keys);
void print_genpaf_align      (FILE* f,
                              seq* seq1, unspos beg1, unspos end1,
                              seq* seq2, unspos beg2, unspos end2,
                              editscript* script, score s,
                              char* keys,
                              unspos idNumer, unspos idDenom,
                              unspos covNumer, unspos covDenom,
                              unspos gapNumer, unspos gapDenom);
void print_genpaf_match      (FILE* f,
                              seq* seq1, unspos pos1,
                              seq* seq2, unspos pos2, unspos length,
                              score s,
                              char* keys);
char* parse_genpaf_keys      (char* s);

#undef global
#endif // genpaf_H
